# Copyright 2020 Open Reaction Database Project Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for ord_schema.scripts.check_pb."""
import docopt
import pytest

from ord_schema import message_helpers
from ord_schema.proto import dataset_pb2
from ord_schema.scripts import check_pb


def test_main_pass(tmp_path):
    pb_filename = (tmp_path / "test.pb").as_posix()
    pbtxt_filename = (tmp_path / "test.pbtxt").as_posix()
    dataset = dataset_pb2.Dataset()
    reaction = dataset.reactions.add()
    component = reaction.inputs["test"].components.add()
    component.identifiers.add(value="c1ccccc1", type="SMILES")
    message_helpers.write_message(dataset, pb_filename)
    message_helpers.write_message(dataset, pbtxt_filename)
    check_pb.main(docopt.docopt(check_pb.__doc__, ["--pb", pb_filename, "--pbtxt", pbtxt_filename]))


def test_main_fail(tmp_path):
    pb_filename = (tmp_path / "test.pb").as_posix()
    pbtxt_filename = (tmp_path / "test.pbtxt").as_posix()
    dataset = dataset_pb2.Dataset()
    reaction = dataset.reactions.add()
    component = reaction.inputs["test"].components.add()
    component.identifiers.add(value="c1ccccc1", type="SMILES")
    message_helpers.write_message(dataset, pb_filename)
    component.identifiers.add(value="benzene", type="NAME")
    message_helpers.write_message(dataset, pbtxt_filename)
    with pytest.raises(ValueError, match="Datasets differ"):
        check_pb.main(docopt.docopt(check_pb.__doc__, ["--pb", pb_filename, "--pbtxt", pbtxt_filename]))



# Copyright 2020 The Open Reaction Database Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Wrappers and utilities for handling protocol buffers in Python."""

import collections
import dataclasses
from typing import Union

import ord_schema

_MESSAGE_TYPES = (
    collections.abc.MutableMapping,  # Proto map.
    ord_schema.Message,  # Generic submessage.
)

# pytype: disable=attribute-error


@dataclasses.dataclass(eq=True, frozen=True)
class FrozenMessage(collections.abc.Mapping):
    """Container for a protocol buffer that does not allow edits.

    Notes:
        * For standard scalar values, it is not possible to distinguish between
          default values and explicitly set values that match the default. If
          the default is a valid value, add the `optional` label to the field.
          See https://github.com/Open-Reaction-Database/ord-schema/pull/174.
        * For `optional` scalar values and all submessage fields, exceptions
          are raised if the user attempts to access an undefined attribute
          (AttributeError), access an undefined map key (KeyError), or set
          any attribute or map value (dataclasses.FrozenInstanceError).
        * I considered adding a `raise_on_error` option that would return None
          instead of raising AttributeError or KeyError when requesting unset
          values. However, this breaks the guarantee that `hasattr` returns
          False for unset `optional` scalar values and submessages.
    """

    _message: Union[_MESSAGE_TYPES]

    def __getattr__(self, name: str):
        """Fetches a message attribute, if it exists.

        Notes:
            * If `name` is a submessage, a new FrozenMessage wrapping the
              submessage is returned.
            * If `name` has not been set explicitly, an AttributeError is
              raised. This allows for distinguishing between default values and
              values that were set explicitly (to the default).

        Args:
            name: Text attribute name.

        Returns:
            The requested attribute. If the attribute is a submessage, a new
            FrozenMessage is returned.

        Raises:
            AttributeError: if `name` has not been set explicitly.
        """
        try:
            if not self._message.HasField(name):
                raise AttributeError(f'attribute "{name}" has not been set')
        except ValueError:
            pass  # The requested attribute is not a submessage.
        value = getattr(self._message, name)
        if isinstance(value, _MESSAGE_TYPES):
            return FrozenMessage(value)
        if hasattr(value, "append"):
            # Make repeated fields (and their elements) immutable.
            repeated_values = []
            for element in value:
                if isinstance(element, _MESSAGE_TYPES):
                    repeated_values.append(FrozenMessage(element))
                else:
                    repeated_values.append(element)
            return tuple(repeated_values)
        return value

    def __iter__(self):
        return self._message.__iter__()

    def __len__(self):
        return len(self._message)

    def __getitem__(self, key):
        """Fetches a message key, if it exists.

        Notes:
            * If `key` is a submessage, a new FrozenMessage wrapping the
              submessage is returned.
            * If `key` has not been set explicitly, an AttributeError is
              raised. This allows for distinguishing between default values and
              values that were set explicitly (to the default).

        Args:
            key: Text key into the underlying message.

        Returns:
            The requested value. If the value is a submessage, a new
            FrozenMessage is returned.

        Raises:
            AttributeError: if `key` has not been set explicitly.
        """
        if key not in self._message:
            raise KeyError(f'key "{key}" has not been set')
        value = self._message[key]
        if isinstance(value, _MESSAGE_TYPES):
            return FrozenMessage(value)
        return value



# Copyright 2020 Open Reaction Database Project Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Validates a set of Dataset protocol buffers.

Usage:
    validate.py --input=<str> [--filter=<str>]

Options:
    --input=<str>       Input pattern for Dataset protos
    --filter=<str>      Regex filename filter
"""
import glob
import re
from collections.abc import Iterable

import docopt
from rdkit import RDLogger

from ord_schema.logging import get_logger
from ord_schema import message_helpers
from ord_schema import validations
from ord_schema.proto import dataset_pb2

logger = get_logger(__name__)


def filter_filenames(filenames: Iterable[str], pattern: str) -> list[str]:
    """Filters filenames according to a regex pattern."""
    filtered_filenames = []
    for filename in filenames:
        if re.search(pattern, filename):
            filtered_filenames.append(filename)
    return filtered_filenames


def main(kwargs):
    filenames = sorted(glob.glob(kwargs["--input"], recursive=True))
    logger.info("Found %d datasets", len(filenames))
    if kwargs["--filter"]:
        filenames = filter_filenames(filenames, kwargs["--filter"])
        logger.info("Filtered to %d datasets", len(filenames))
    for filename in filenames:
        logger.info("Validating %s", filename)
        dataset = message_helpers.load_message(filename, dataset_pb2.Dataset)
        validations.validate_datasets({filename: dataset})


if __name__ == "__main__":
    RDLogger.DisableLog("rdApp.*")  # Disable RDKit logging.
    main(docopt.docopt(__doc__))



# Copyright 2020 Open Reaction Database Project Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for ord_schema.scripts.enumerate_dataset."""
import os

import docopt
import pandas as pd
import pytest

from ord_schema import message_helpers
from ord_schema import validations
from ord_schema.proto import dataset_pb2
from ord_schema.proto import reaction_pb2
from ord_schema.scripts import enumerate_dataset


@pytest.fixture
def setup(tmp_path) -> tuple[str, str, str]:
    dirname = tmp_path.as_posix()
    template_string = """
    inputs {
        key: "test"
        value {
            components {
                identifiers {
                    type: SMILES
                    value: "$input_smiles$"
                }
                amount {
                    mass {
                        value: $input_mass$
                        units: GRAM
                    }
                }
            }
        }
    }
    outcomes {
        analyses {
            key: "my_analysis"
            value {
                type: WEIGHT
            }
        }
        products {
            identifiers {
                type: SMILES
                value: "$product_smiles$"
            }
            measurements {
                analysis_key: "my_analysis"
                type: YIELD
                percentage {
                    value: $product_yield$
                }
            }
        }
    }
    """
    template_filename = os.path.join(dirname, "template.pbtxt")
    with open(template_filename, "w") as f:
        f.write(template_string)
    data = pd.DataFrame(
        {
            "input_smiles": ["C", "CC", "CCC"],
            "input_mass": [1.2, 3.4, 5.6],
            "product_smiles": ["CO", "CCO", "CCCO"],
            "product_yield": [7.8, 9.0, 8.7],
        }
    )
    spreadsheet_filename = os.path.join(dirname, "spreadsheet.csv")
    data.to_csv(spreadsheet_filename, index=False)
    expected = dataset_pb2.Dataset()
    reaction1 = expected.reactions.add()
    reaction1_compound1 = reaction1.inputs["test"].components.add()
    reaction1_compound1.identifiers.add(value="C", type="SMILES")
    reaction1_compound1.amount.mass.CopyFrom(reaction_pb2.Mass(value=1.2, units="GRAM"))
    reaction1_product1 = reaction1.outcomes.add().products.add()
    reaction1_product1.identifiers.add(value="CO", type="SMILES")
    reaction1_product1.measurements.add(analysis_key="my_analysis", type="YIELD", percentage=dict(value=7.8))
    reaction1.outcomes[0].analyses["my_analysis"].type = reaction_pb2.Analysis.WEIGHT
    reaction2 = expected.reactions.add()
    reaction2_compound1 = reaction2.inputs["test"].components.add()
    reaction2_compound1.identifiers.add(value="CC", type="SMILES")
    reaction2_compound1.amount.mass.CopyFrom(reaction_pb2.Mass(value=3.4, units="GRAM"))
    reaction2_product1 = reaction2.outcomes.add().products.add()
    reaction2_product1.identifiers.add(value="CCO", type="SMILES")
    reaction2_product1.measurements.add(analysis_key="my_analysis", type="YIELD", percentage=dict(value=9.0))
    reaction2.outcomes[0].analyses["my_analysis"].type = reaction_pb2.Analysis.WEIGHT
    reaction3 = expected.reactions.add()
    reaction3_compound1 = reaction3.inputs["test"].components.add()
    reaction3_compound1.identifiers.add(value="CCC", type="SMILES")
    reaction3_compound1.amount.mass.CopyFrom(reaction_pb2.Mass(value=5.6, units="GRAM"))
    reaction3_product1 = reaction3.outcomes.add().products.add()
    reaction3_product1.identifiers.add(value="CCCO", type="SMILES")
    reaction3_product1.measurements.add(analysis_key="my_analysis", type="YIELD", percentage=dict(value=8.7))
    reaction3.outcomes[0].analyses["my_analysis"].type = reaction_pb2.Analysis.WEIGHT
    yield template_filename, spreadsheet_filename, expected


def test_main(setup, tmp_path):
    template_filename, spreadsheet_filename, expected = setup
    output_filename = (tmp_path / "dataset.pbtxt").as_posix()
    argv = [
        "--template",
        template_filename,
        "--spreadsheet",
        spreadsheet_filename,
        "--output",
        output_filename,
    ]
    enumerate_dataset.main(docopt.docopt(enumerate_dataset.__doc__, argv))
    assert os.path.exists(output_filename)
    dataset = message_helpers.load_message(output_filename, dataset_pb2.Dataset)
    assert len(dataset.reactions) == 3
    validations.validate_message(dataset, raise_on_error=True)
    assert dataset == expected



# Copyright 2020 Open Reaction Database Project Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Helpers for translating strings with units."""

import re
from typing import Optional, Type

import numpy as np

import ord_schema
from ord_schema.proto import reaction_pb2

# Accepted synonyms for units. Note that all values will be converted to
# lowercase.
_UNIT_SYNONYMS = {
    reaction_pb2.Time: {
        reaction_pb2.Time.DAY: ["d", "day", "days"],
        reaction_pb2.Time.HOUR: ["h", "hour", "hours", "hr", "hrs"],
        reaction_pb2.Time.MINUTE: ["min", "mins", "minute", "minutes"],
        reaction_pb2.Time.SECOND: ["s", "sec", "secs", "second", "seconds"],
    },
    reaction_pb2.Mass: {
        reaction_pb2.Mass.GRAM: ["g", "gram", "grams", "gs", "gm", "gms", "gr"],
        reaction_pb2.Mass.MILLIGRAM: ["mg", "mgs", "milligrams", "milligram"],
        reaction_pb2.Mass.MICROGRAM: [
            "μg",
            "ug",
            "ugs",
            "micg",
            "micgs",
            "micrograms",
            "microgram",
        ],
        reaction_pb2.Mass.KILOGRAM: ["kg", "kgs", "kilogram", "kilograms"],
    },
    reaction_pb2.Moles: {
        reaction_pb2.Moles.MOLE: ["mol", "mols", "mole", "moles"],
        reaction_pb2.Moles.MILLIMOLE: [
            "mmol",
            "millimoles",
            "mmols",
            "mmole",
            "mmoles",
        ],
        reaction_pb2.Moles.MICROMOLE: ["μmol", "umol", "umols", "micromoles"],
        reaction_pb2.Moles.NANOMOLE: ["nmol", "nanomoles"],
    },
    reaction_pb2.Volume: {
        reaction_pb2.Volume.MILLILITER: [
            "mL",
            "milliliter",
            "milliliters",
            "cc",
            "cm3",
            "mls",
        ],
        reaction_pb2.Volume.MICROLITER: [
            "μL",
            "uL",
            "micl",
            "microliter",
            "microliters",
        ],
        reaction_pb2.Volume.LITER: ["L", "liter", "liters", "litres"],
        reaction_pb2.Volume.NANOLITER: ["nL", "nanoliter", "nanoliters"],
    },
    reaction_pb2.Length: {
        reaction_pb2.Length.CENTIMETER: ["cm", "centimeter"],
        reaction_pb2.Length.MILLIMETER: ["millimeter", "millimeters"],
        reaction_pb2.Length.METER: ["meter", "meters"],
        reaction_pb2.Length.INCH: ["in", "inch", "inches"],
        reaction_pb2.Length.FOOT: ["ft", "foot", "feet"],
    },
    reaction_pb2.Pressure: {
        reaction_pb2.Pressure.BAR: ["bar", "barg", "bars"],
        reaction_pb2.Pressure.ATMOSPHERE: ["atm", "atmosphere", "atmospheres"],
        reaction_pb2.Pressure.PSI: ["psi"],
        reaction_pb2.Pressure.KPSI: ["kpsi"],
        reaction_pb2.Pressure.PASCAL: ["Pa", "pascal", "pascals", "pas"],
        reaction_pb2.Pressure.KILOPASCAL: ["kPa", "kilopascals", "kPas"],
    },
    reaction_pb2.Temperature: {
        reaction_pb2.Temperature.CELSIUS: [
            "°C",
            "C",
            "degC",
            "°celsius",
            "celsius",
            "degrees C",
        ],
        reaction_pb2.Temperature.FAHRENHEIT: ["°F", "F", "degF", "fahrenheit"],
        reaction_pb2.Temperature.KELVIN: ["K", "degK", "Kelvin"],
    },
    reaction_pb2.Current: {
        reaction_pb2.Current.AMPERE: ["A", "ampere", "amps", "amp"],
        reaction_pb2.Current.MILLIAMPERE: [
            "mA",
            "milliampere",
            "milliamp",
            "milliamps",
        ],
    },
    reaction_pb2.Voltage: {
        reaction_pb2.Voltage.VOLT: ["V", "volt", "volts"],
        reaction_pb2.Voltage.MILLIVOLT: ["mV", "millivolt", "millivolts"],
    },
    reaction_pb2.Wavelength: {
        reaction_pb2.Wavelength.NANOMETER: ["nm", "nanometer", "nanometers"],
        reaction_pb2.Wavelength.WAVENUMBER: [
            "cm⁻¹",
            "cm^-1",
            "cm-1",
            "wavenumber",
            "1/cm",
        ],
    },
    reaction_pb2.FlowRate: {
        reaction_pb2.FlowRate.MICROLITER_PER_MINUTE: ["μL/min", "uL/min"],
        reaction_pb2.FlowRate.MICROLITER_PER_SECOND: ["μL/s", "uL/s"],
        reaction_pb2.FlowRate.MILLILITER_PER_MINUTE: ["mL/min"],
        reaction_pb2.FlowRate.MILLILITER_PER_SECOND: ["mL/s"],
        reaction_pb2.FlowRate.MICROLITER_PER_HOUR: ["μL/h", "uL/h"],
    },
}

_FORBIDDEN_UNITS = {
    "m": "ambiguous between meter and minute",
}

# Concentration units are defined separately since they are not needed for any
# native fields in the reaction schema.
CONCENTRATION_UNIT_SYNONYMS = {
    reaction_pb2.Concentration: {
        reaction_pb2.Concentration.MOLAR: ["M", "molar"],
        reaction_pb2.Concentration.MILLIMOLAR: ["mM", "millimolar"],
        reaction_pb2.Concentration.MICROMOLAR: ["uM", "micromolar"],
    },
}

VOLUME_L_PER_UNIT = {
    reaction_pb2.Volume.VolumeUnit.LITER: 1,
    reaction_pb2.Volume.VolumeUnit.MILLILITER: 1e-3,
    reaction_pb2.Volume.VolumeUnit.MICROLITER: 1e-6,
    reaction_pb2.Volume.VolumeUnit.NANOLITER: 1e-9,
}

CONCENTRATION_M_PER_UNIT = {
    reaction_pb2.Concentration.ConcentrationUnit.MOLAR: 1,
    reaction_pb2.Concentration.ConcentrationUnit.MILLIMOLAR: 1e-3,
    reaction_pb2.Concentration.ConcentrationUnit.MICROMOLAR: 1e-6,
}


class UnitResolver:
    """Resolver class for translating value+unit strings into messages."""

    def __init__(
        self,
        unit_synonyms: Optional[dict[Type[ord_schema.UnitMessage], dict[ord_schema.Message, list[str]]]] = None,
        forbidden_units: Optional[dict[str, str]] = None,
    ):
        """Initializes a UnitResolver.

        Args:
            unit_synonyms: A dictionary of dictionaries that defines, for each
                message type (first key) and for each unit option (second key),
                a list of strings that defines how that unit can be written.
                Defaults to None. If None, uses default _UNIT_SYNONYMS dict.
            forbidden_units: A dictionary where each key is a string that is a
                plausible way of writing a unit and a value explaining why
                the UnitResolver cannot resolve that unit. The prototypical
                case is one of ambiguity (e.g., "m" can mean meter or minute).
                Defaults to None. If None, uses default _FORBIDDEN_UNITS dict.
                If no units are forbidden, an empty dictionary should be used.
        """
        if unit_synonyms is None:
            unit_synonyms = _UNIT_SYNONYMS
        if forbidden_units is None:
            forbidden_units = _FORBIDDEN_UNITS
        self._forbidden_units = forbidden_units
        self._resolver = {}
        for message in unit_synonyms:
            for unit in unit_synonyms[message]:
                for string_unit in unit_synonyms[message][unit]:
                    string_unit = string_unit.lower()
                    if string_unit in self._resolver:
                        raise KeyError(f"duplicated unit: {string_unit}")
                    self._resolver[string_unit] = (message, unit)
        # Values must have zero or one decimal point. Whitespace between the
        # value and the unit is optional.
        self._pattern = re.compile(r"(-?\d+\.?\d*(?:[eE]-?\d+)?)(?:[-±](-?\d+\.?\d*))?\s*([\w\sμ°]+)\.?")

    def resolve(self, string: str, allow_range: bool = False) -> ord_schema.UnitMessage:
        """Resolves a string into a message containing a value with units.

        Args:
            string: The string to parse; must contain a numeric value and a
                string unit. For example: "1.25 h".
            allow_range: If True, ranges like "1-2 h" can be provided and the
                average value will be reported along with the standard
                deviation.

        Returns:
            Message containing a numeric value with units listed in the schema.

        Raises:
            ValueError: if string does not contain a value with units, or if
                the value is invalid.
        """
        # NOTE(kearnes): Use fullmatch() to catch cases with multiple matches.
        match = self._pattern.fullmatch(string.strip().replace("−", "-"))
        if not match:
            raise ValueError(f"string does not contain a value with units: {string}")
        value, range_value, string_unit = match.groups()
        precision = None
        if range_value is not None:
            if "±" in string:
                value = float(value)
                precision = float(range_value)
            elif not allow_range:
                raise ValueError("string appears to contain a range of values " f"but allow_range is False: {string}")
            else:
                values = np.asarray([value, range_value], dtype=float)
                value = values.mean()
                precision = values.std()
        else:
            value = float(value)
        assert string_unit is not None  # Type hint.
        string_unit = string_unit.lower()
        if string_unit in self._forbidden_units:
            raise KeyError(f"forbidden units: {string_unit}: " f"({self._forbidden_units[string_unit]})")
        if string_unit not in self._resolver:
            raise KeyError(f"unrecognized units: {string_unit}")
        message, unit = self._resolver[string_unit]
        if value < 0.0 and message != reaction_pb2.Temperature:
            raise ValueError(f"negative values are only allowed for temperature: {string}")
        if precision:
            return message(value=value, precision=precision, units=unit)
        return message(value=value, units=unit)


def format_message(message: ord_schema.UnitMessage) -> Optional[str]:
    """Formats a united message into a string.

    Args:
        message: a message with units, e.g., Mass, Length.

    Returns:
        A string describing the value, e.g., "5.0 (p/m 0.1) mL" using the
            first unit synonym listed in _UNIT_SYNONYMS.
    """
    if message.units == getattr(type(message)(), "UNSPECIFIED"):
        return None
    txt = f"{message.value:.7g} "
    if message.precision:
        txt += f"(± {message.precision:.7g}) "
    txt += _UNIT_SYNONYMS[type(message)][message.units][0]
    return txt


def compute_solute_quantity(
    volume: reaction_pb2.Volume, concentration: reaction_pb2.Concentration
) -> reaction_pb2.Amount:
    """Computes the quantity of a solute, given volume and concentration."""
    volume_conversion = VOLUME_L_PER_UNIT[volume.units]
    volume_liter = volume.value * volume_conversion
    concentration_conversion = CONCENTRATION_M_PER_UNIT[concentration.units]
    concentration_molar = concentration.value * concentration_conversion

    moles = volume_liter * concentration_molar
    if moles < 1e-6:
        value = moles * 1e9
        unit = reaction_pb2.Moles.NANOMOLE
    elif moles < 1e-3:
        value = moles * 1e6
        unit = reaction_pb2.Moles.MICROMOLE
    elif moles < 1:
        value = moles * 1e3
        unit = reaction_pb2.Moles.MILLIMOLE
    else:
        value = moles
        unit = reaction_pb2.Moles.MOLE
    return reaction_pb2.Amount(moles=reaction_pb2.Moles(value=value, units=unit))



# Copyright 2022 Open Reaction Database Project Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for ord_schema.orm.database."""
from sqlalchemy import select

from ord_schema.orm.database import delete_dataset
from ord_schema.orm.mappers import Mappers
from ord_schema.proto import reaction_pb2


def test_orm(test_session):
    query = (
        select(Mappers.Reaction)
        .join(Mappers.ReactionOutcome)
        .join(Mappers.ProductCompound)
        .join(Mappers.ProductMeasurement)
        .join(Mappers.Percentage)
        .where(Mappers.ProductMeasurement.type == "YIELD", Mappers.Percentage.value >= 70)
    )
    results = test_session.execute(query)
    reactions = [reaction_pb2.Reaction.FromString(result[0].proto) for result in results]
    assert len(reactions) == 12


def test_delete_dataset(test_session):
    assert test_session.query(Mappers.Reaction).count() == 80
    delete_dataset("test_dataset", test_session)
    assert test_session.query(Mappers.Reaction).count() == 0



# Copyright 2023 Open Reaction Database Project Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ord-schema/proto/test.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1bord-schema/proto/test.proto\x12\x08ord_test\"\xaf\x01\n\x06Scalar\x12\x13\n\x0bint32_value\x18\x01 \x01(\x05\x12\x13\n\x0bint64_value\x18\x02 \x01(\x03\x12\x18\n\x0b\x66loat_value\x18\x03 \x01(\x02H\x00\x88\x01\x01\x12\x14\n\x0cstring_value\x18\x04 \x01(\t\x12\x13\n\x0b\x62ytes_value\x18\x05 \x01(\x0c\x12\x17\n\nbool_value\x18\x06 \x01(\x08H\x01\x88\x01\x01\x42\x0e\n\x0c_float_valueB\r\n\x0b_bool_value\" \n\x0eRepeatedScalar\x12\x0e\n\x06values\x18\x01 \x03(\x02\"f\n\x04\x45num\x12(\n\x05value\x18\x01 \x01(\x0e\x32\x19.ord_test.Enum.EnumValues\"4\n\nEnumValues\x12\x0f\n\x0bUNSPECIFIED\x10\x00\x12\t\n\x05\x46IRST\x10\x01\x12\n\n\x06SECOND\x10\x02\"w\n\x0cRepeatedEnum\x12\x31\n\x06values\x18\x01 \x03(\x0e\x32!.ord_test.RepeatedEnum.EnumValues\"4\n\nEnumValues\x12\x0f\n\x0bUNSPECIFIED\x10\x00\x12\t\n\x05\x46IRST\x10\x01\x12\n\n\x06SECOND\x10\x02\"V\n\x06Nested\x12%\n\x05\x63hild\x18\x01 \x01(\x0b\x32\x16.ord_test.Nested.Child\x1a%\n\x05\x43hild\x12\x12\n\x05value\x18\x01 \x01(\x02H\x00\x88\x01\x01\x42\x08\n\x06_value\"i\n\x0eRepeatedNested\x12\x30\n\x08\x63hildren\x18\x01 \x03(\x0b\x32\x1e.ord_test.RepeatedNested.Child\x1a%\n\x05\x43hild\x12\x12\n\x05value\x18\x01 \x01(\x02H\x00\x88\x01\x01\x42\x08\n\x06_value\"_\n\x03Map\x12)\n\x06values\x18\x01 \x03(\x0b\x32\x19.ord_test.Map.ValuesEntry\x1a-\n\x0bValuesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x02:\x02\x38\x01\"\xb3\x01\n\tMapNested\x12\x33\n\x08\x63hildren\x18\x01 \x03(\x0b\x32!.ord_test.MapNested.ChildrenEntry\x1a%\n\x05\x43hild\x12\x12\n\x05value\x18\x01 \x01(\x02H\x00\x88\x01\x01\x42\x08\n\x06_value\x1aJ\n\rChildrenEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12(\n\x05value\x18\x02 \x01(\x0b\x32\x19.ord_test.MapNested.Child:\x02\x38\x01\x62\x06proto3')



_SCALAR = DESCRIPTOR.message_types_by_name['Scalar']
_REPEATEDSCALAR = DESCRIPTOR.message_types_by_name['RepeatedScalar']
_ENUM = DESCRIPTOR.message_types_by_name['Enum']
_REPEATEDENUM = DESCRIPTOR.message_types_by_name['RepeatedEnum']
_NESTED = DESCRIPTOR.message_types_by_name['Nested']
_NESTED_CHILD = _NESTED.nested_types_by_name['Child']
_REPEATEDNESTED = DESCRIPTOR.message_types_by_name['RepeatedNested']
_REPEATEDNESTED_CHILD = _REPEATEDNESTED.nested_types_by_name['Child']
_MAP = DESCRIPTOR.message_types_by_name['Map']
_MAP_VALUESENTRY = _MAP.nested_types_by_name['ValuesEntry']
_MAPNESTED = DESCRIPTOR.message_types_by_name['MapNested']
_MAPNESTED_CHILD = _MAPNESTED.nested_types_by_name['Child']
_MAPNESTED_CHILDRENENTRY = _MAPNESTED.nested_types_by_name['ChildrenEntry']
_ENUM_ENUMVALUES = _ENUM.enum_types_by_name['EnumValues']
_REPEATEDENUM_ENUMVALUES = _REPEATEDENUM.enum_types_by_name['EnumValues']
Scalar = _reflection.GeneratedProtocolMessageType('Scalar', (_message.Message,), {
  'DESCRIPTOR' : _SCALAR,
  '__module__' : 'ord_schema.proto.test_pb2'
  # @@protoc_insertion_point(class_scope:ord_test.Scalar)
  })
_sym_db.RegisterMessage(Scalar)

RepeatedScalar = _reflection.GeneratedProtocolMessageType('RepeatedScalar', (_message.Message,), {
  'DESCRIPTOR' : _REPEATEDSCALAR,
  '__module__' : 'ord_schema.proto.test_pb2'
  # @@protoc_insertion_point(class_scope:ord_test.RepeatedScalar)
  })
_sym_db.RegisterMessage(RepeatedScalar)

Enum = _reflection.GeneratedProtocolMessageType('Enum', (_message.Message,), {
  'DESCRIPTOR' : _ENUM,
  '__module__' : 'ord_schema.proto.test_pb2'
  # @@protoc_insertion_point(class_scope:ord_test.Enum)
  })
_sym_db.RegisterMessage(Enum)

RepeatedEnum = _reflection.GeneratedProtocolMessageType('RepeatedEnum', (_message.Message,), {
  'DESCRIPTOR' : _REPEATEDENUM,
  '__module__' : 'ord_schema.proto.test_pb2'
  # @@protoc_insertion_point(class_scope:ord_test.RepeatedEnum)
  })
_sym_db.RegisterMessage(RepeatedEnum)

Nested = _reflection.GeneratedProtocolMessageType('Nested', (_message.Message,), {

  'Child' : _reflection.GeneratedProtocolMessageType('Child', (_message.Message,), {
    'DESCRIPTOR' : _NESTED_CHILD,
    '__module__' : 'ord_schema.proto.test_pb2'
    # @@protoc_insertion_point(class_scope:ord_test.Nested.Child)
    })
  ,
  'DESCRIPTOR' : _NESTED,
  '__module__' : 'ord_schema.proto.test_pb2'
  # @@protoc_insertion_point(class_scope:ord_test.Nested)
  })
_sym_db.RegisterMessage(Nested)
_sym_db.RegisterMessage(Nested.Child)

RepeatedNested = _reflection.GeneratedProtocolMessageType('RepeatedNested', (_message.Message,), {

  'Child' : _reflection.GeneratedProtocolMessageType('Child', (_message.Message,), {
    'DESCRIPTOR' : _REPEATEDNESTED_CHILD,
    '__module__' : 'ord_schema.proto.test_pb2'
    # @@protoc_insertion_point(class_scope:ord_test.RepeatedNested.Child)
    })
  ,
  'DESCRIPTOR' : _REPEATEDNESTED,
  '__module__' : 'ord_schema.proto.test_pb2'
  # @@protoc_insertion_point(class_scope:ord_test.RepeatedNested)
  })
_sym_db.RegisterMessage(RepeatedNested)
_sym_db.RegisterMessage(RepeatedNested.Child)

Map = _reflection.GeneratedProtocolMessageType('Map', (_message.Message,), {

  'ValuesEntry' : _reflection.GeneratedProtocolMessageType('ValuesEntry', (_message.Message,), {
    'DESCRIPTOR' : _MAP_VALUESENTRY,
    '__module__' : 'ord_schema.proto.test_pb2'
    # @@protoc_insertion_point(class_scope:ord_test.Map.ValuesEntry)
    })
  ,
  'DESCRIPTOR' : _MAP,
  '__module__' : 'ord_schema.proto.test_pb2'
  # @@protoc_insertion_point(class_scope:ord_test.Map)
  })
_sym_db.RegisterMessage(Map)
_sym_db.RegisterMessage(Map.ValuesEntry)

MapNested = _reflection.GeneratedProtocolMessageType('MapNested', (_message.Message,), {

  'Child' : _reflection.GeneratedProtocolMessageType('Child', (_message.Message,), {
    'DESCRIPTOR' : _MAPNESTED_CHILD,
    '__module__' : 'ord_schema.proto.test_pb2'
    # @@protoc_insertion_point(class_scope:ord_test.MapNested.Child)
    })
  ,

  'ChildrenEntry' : _reflection.GeneratedProtocolMessageType('ChildrenEntry', (_message.Message,), {
    'DESCRIPTOR' : _MAPNESTED_CHILDRENENTRY,
    '__module__' : 'ord_schema.proto.test_pb2'
    # @@protoc_insertion_point(class_scope:ord_test.MapNested.ChildrenEntry)
    })
  ,
  'DESCRIPTOR' : _MAPNESTED,
  '__module__' : 'ord_schema.proto.test_pb2'
  # @@protoc_insertion_point(class_scope:ord_test.MapNested)
  })
_sym_db.RegisterMessage(MapNested)
_sym_db.RegisterMessage(MapNested.Child)
_sym_db.RegisterMessage(MapNested.ChildrenEntry)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _MAP_VALUESENTRY._options = None
  _MAP_VALUESENTRY._serialized_options = b'8\001'
  _MAPNESTED_CHILDRENENTRY._options = None
  _MAPNESTED_CHILDRENENTRY._serialized_options = b'8\001'
  _SCALAR._serialized_start=42
  _SCALAR._serialized_end=217
  _REPEATEDSCALAR._serialized_start=219
  _REPEATEDSCALAR._serialized_end=251
  _ENUM._serialized_start=253
  _ENUM._serialized_end=355
  _ENUM_ENUMVALUES._serialized_start=303
  _ENUM_ENUMVALUES._serialized_end=355
  _REPEATEDENUM._serialized_start=357
  _REPEATEDENUM._serialized_end=476
  _REPEATEDENUM_ENUMVALUES._serialized_start=303
  _REPEATEDENUM_ENUMVALUES._serialized_end=355
  _NESTED._serialized_start=478
  _NESTED._serialized_end=564
  _NESTED_CHILD._serialized_start=527
  _NESTED_CHILD._serialized_end=564
  _REPEATEDNESTED._serialized_start=566
  _REPEATEDNESTED._serialized_end=671
  _REPEATEDNESTED_CHILD._serialized_start=527
  _REPEATEDNESTED_CHILD._serialized_end=564
  _MAP._serialized_start=673
  _MAP._serialized_end=768
  _MAP_VALUESENTRY._serialized_start=723
  _MAP_VALUESENTRY._serialized_end=768
  _MAPNESTED._serialized_start=771
  _MAPNESTED._serialized_end=950
  _MAPNESTED_CHILD._serialized_start=527
  _MAPNESTED_CHILD._serialized_end=564
  _MAPNESTED_CHILDRENENTRY._serialized_start=876
  _MAPNESTED_CHILDRENENTRY._serialized_end=950
# @@protoc_insertion_point(module_scope)



# Copyright 2022 Open Reaction Database Project Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Logging utilities."""
import logging


def get_logger(name: str, level: int = logging.INFO) -> logging.Logger:
    """Creates a Logger."""
    if not get_logger.initialized:
        logging.basicConfig(format="%(levelname)s %(asctime)s %(filename)s:%(lineno)d: %(message)s")
        get_logger.initialized = True
    logger = logging.getLogger(name)
    logger.setLevel(level)
    return logger


get_logger.initialized = False



# Copyright 2023 Open Reaction Database Project Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ord-schema/proto/dataset.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ord_schema.proto import reaction_pb2 as ord__schema_dot_proto_dot_reaction__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1eord-schema/proto/dataset.proto\x12\x03ord\x1a\x1ford-schema/proto/reaction.proto\"x\n\x07\x44\x61taset\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12 \n\treactions\x18\x03 \x03(\x0b\x32\r.ord.Reaction\x12\x14\n\x0creaction_ids\x18\x04 \x03(\t\x12\x12\n\ndataset_id\x18\x05 \x01(\t\"i\n\x0e\x44\x61tasetExample\x12\x12\n\ndataset_id\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x0b\n\x03url\x18\x03 \x01(\t\x12!\n\x07\x63reated\x18\x04 \x01(\x0b\x32\x10.ord.RecordEventb\x06proto3')



_DATASET = DESCRIPTOR.message_types_by_name['Dataset']
_DATASETEXAMPLE = DESCRIPTOR.message_types_by_name['DatasetExample']
Dataset = _reflection.GeneratedProtocolMessageType('Dataset', (_message.Message,), {
  'DESCRIPTOR' : _DATASET,
  '__module__' : 'ord_schema.proto.dataset_pb2'
  # @@protoc_insertion_point(class_scope:ord.Dataset)
  })
_sym_db.RegisterMessage(Dataset)

DatasetExample = _reflection.GeneratedProtocolMessageType('DatasetExample', (_message.Message,), {
  'DESCRIPTOR' : _DATASETEXAMPLE,
  '__module__' : 'ord_schema.proto.dataset_pb2'
  # @@protoc_insertion_point(class_scope:ord.DatasetExample)
  })
_sym_db.RegisterMessage(DatasetExample)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _DATASET._serialized_start=72
  _DATASET._serialized_end=192
  _DATASETEXAMPLE._serialized_start=194
  _DATASETEXAMPLE._serialized_end=299
# @@protoc_insertion_point(module_scope)
